@@ -1,12 +1,13 @@
 package OpenInteract2::Manage::Package;
 
-# $Id: Package.pm,v 1.14 2004/02/17 04:30:20 lachoy Exp $
+# $Id: Package.pm,v 1.17 2005/03/17 14:58:02 sjn Exp $
 
 use strict;
 use base qw( OpenInteract2::Manage );
+use File::Spec::Functions    qw( catfile );
 use OpenInteract2::Exception qw( oi_error );
 
-$OpenInteract2::Manage::Package::VERSION = sprintf("%d.%02d", q$Revision: 1.14 $ =~ /(\d+)\.(\d+)/);
+$OpenInteract2::Manage::Package::VERSION = sprintf("%d.%02d", q$Revision: 1.17 $ =~ /(\d+)\.(\d+)/);
 
 ########################################
 # INTERFACE
@@ -34,6 +35,19 @@ sub validate_param {
     return $self->SUPER::validate_param( $name, $value );
 }
 
+# We only want to see if 'package_dir' contains a package if the
+# 'package' parameter is not defined
+
+sub _check_package_dir {
+    my ( $self, $package_dir ) = @_;
+    unless ( -d $package_dir ) {
+        return "Value given for 'package_dir' ($package_dir) is not " .
+               "a valid directory";
+    }
+    my $packages = $self->param( 'package' ) || [];
+    return if ( scalar @{ $packages } > 0 );
+    return $self->_package_in_dir( $package_dir );
+}
 
 # Test whether directory $package_dir is actually a package directory
 # - returns message on error, nothing otherwise.
@@ -42,13 +56,13 @@ sub _package_in_dir {
     my ( $self, $package_dir ) = @_;
     eval { opendir( PKGDIR, $package_dir ) || die $! };
     if ( $@ ) {
-        return "Cannot open directory [$package_dir]: $@";
+        return "Cannot open directory '$package_dir': $@";
     }
     my %pkg_files = map { $_ => 1 }
-                    grep { -f File::Spec->catfile( $package_dir, $_ ) }
+                    grep { -f catfile( $package_dir, $_ ) }
                     readdir( PKGDIR );
-    unless ( $pkg_files{'package.conf'} ) {
-        return "Directory [$package_dir] does not contain a package";
+    unless ( $pkg_files{'package.ini'} ) {
+        return "Directory '$package_dir' does not contain a package";
     }
     return;
 }
@@ -80,7 +94,7 @@ Returns: arrayref of package names.
 
 =head1 COPYRIGHT
 
-Copyright (c) 2002-2004 Chris Winters. All rights reserved.
+Copyright (c) 2002-2005 Chris Winters. All rights reserved.
 
 This library is free software; you can redistribute it and/or modify
 it under the same terms as Perl itself.
